function [ampEst] = BIAA(data, freqPts, phi, NVec, iterMax, alg)

% [ampEst] = BIAA(data, freqPts, phi, NVec, iterMax, alg)
%
% Estimate the amplitude spectrum of a medical ultrasound signal using IAA. The
% algorithm is published in:
%   Gudmundson et al.: "Blood Velocity Estimation Using Ultrasound and
%   Spectral Iterative Adaptive Approaches", Signal Processing, 2011.
%   DOI: 	10.1016/j.sigpro.2010.12.014
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% IN:
%   data    : KxN matrix of data, where K is the number of samples along 
%             depth (fast-time samples) and N is the number of emissions 
%             (slow-time samples).
%   freqPts : The number of frequency points in which to compute the 
%             amplitude spectrum.
%   phi     : Phase shift due to depth.
%   NVec    : Vector pointing out the time of the available samples.
%   iterMax : Number of iterations for IAA.
%   alg     : 1-Average over fast-time data
%             2-Average over cov. matrix (Recommended)
%             3-Average of amplitudes (very slow)
%
% OUT:
%   ampEst  : Amplitude estimate.
%
% Erik Gudmundson, 2009-12-29
%

NVec = NVec(:);
dataTmp = data(:,NVec);
[K,N] = size(dataTmp);

%Generate the demodulating matrix
Dinv = repmat(exp(-i*phi*(0:K-1)).',1,N);

%Get the demodulated data
z = Dinv.*dataTmp;
%We want the data as col. vectors
z = transpose(z);

%Generate the 'Fourier matrix'
psiVec = 2*pi*(0:freqPts-1)'/freqPts;
A = exp(i*NVec*psiVec.');

%The mean of the data
zMean = mean(z,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize using least squares
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (alg == 2 || alg == 3)
  ampEstMat = A'*z/N; %Individual estimates per fast-time
else %Here, alg == 1
  ampEst = A'*zMean/N;
end
if (alg == 2 || alg == 3)
  eta = 1e-10;
else %Here, alg == 1
  eta = 1e-10*ones(1,K);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do the iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if alg == 3
  eta = repmat(eta,1,K);
  for iter = 1:iterMax
    for k = 1:K
      R_k = A*diag( conj(ampEstMat(:,k)).*ampEstMat(:,k) )*A' + eta(k)*eye(N);
      R_kInv = inv(R_k);
      ampEstMat(:,k) = A'*R_kInv*z(:,k)./diag(A'*R_kInv*A);
      
      eta(k) = mean(abs(R_kInv*z(:,k)./diag(R_kInv)).^2);
    end
  end
  ampEst = mean(ampEstMat,2);
  
elseif alg == 2
  for iter = 1:iterMax
    R = A*diag(mean(conj(ampEstMat).*ampEstMat,2))*A' + eta*eye(N);
    
    RInv = inv(R);
    ampEstMat = A'*RInv*z./repmat(diag(A'*RInv*A),1,K);
    
    eta = zeros(1,K);
    for k=1:K
      eta(k) = mean(abs(RInv*z(:,k)./diag(RInv)).^2);
    end
    eta = mean(eta);
    
  end
  
  %The final step
  ampEst = mean(ampEstMat,2);
  
elseif alg == 1
  for iter = 1:iterMax
    %Covariance est
    R = A*diag(conj(ampEst).*ampEst)*A' + mean(eta)*eye(N);
    RInv = inv(R);
    ampEst = A'*RInv*zMean./diag(A'*RInv*A);
    eta = abs( RInv*zMean./diag(RInv) ).^2;
  end
end



