function [f,Rr] = pg_iaa_fast_qn(xg,N,S,q,K,m,qn)
% [f,Rr] = pg_iaa_fast_qn(xg,N,S,q,K,m,qn)
%
% Fast Periodically Gapped IAA, with a Quasi-Newton algorithm.
%
% This function is also used for initialization of pg_iaa_fast_1_iter_qn().
%
% Computational efficient implementation of the BIAA algorithm that can be
% used in the scenario of periodically gapped data. This means that data is
% available in a specific repeated pattern, for example [1 1 1 0 0 1 1 1 0
% 0]. This example then has two blocks of pattern [1 1 1 0 0] where '1'
% means data is available and '0' that the sample is missing.
%
% This method, referred to as BR-QN-F-BIAA_2, was first published in:
%   A. Jakobsson, G. O. Glentis and E. Gudmundson, ''Computationally
%   Efficient Time-Recursive IAA-Based Blood Velocity Estimation'', IEEE
%   Transactions on Signal Processing.
%
% IN:
%  xg : vector of available data (ie. with missing data removed!)
%  N  : total time window, ie. number of samples incl. the missing samples
%  S  : data segment size, ie. length of each block of data incl. missing samples
%  q  : number of available data elements within a data segment
%  K  : number of frequency points at which to compute the spectrum
%  m  : number of IAA iterations
%  qn : order of AR approximation (must be smaller than the number of
%       sub-blocks used, ie. N/S)
%
% OUT:
%  f   : the spectrum
%  Rr  : first block column of R
%
% G.O. Glentis, UoP 2010
%

seg=S;
kn=q;
kg=N/S; % number of segments
Ng=kg*q;
n=[0:N-1]';
%
%  Create gap pattern
mg=[];
for i=1:kg
  mg=[mg;ones(q,1);zeros(S-q,1)];
end
mg=logical(mg);
ng=n(mg);

c=xg;

for k=1:m
  xxg=c;
  xxgn=zeros(N,1);
  xxgn(mg)=xxg;
  giaa_num=fft([xxgn;zeros(K-N,1)]);
  if(k==1),
    giaa_den=ones(K,1)*Ng;
  else
    fa1=pgap_coef_gs(Ac,Bc,kn,kg,N,seg);
    Fa=fft([fa1;zeros(K-2*N+1,1)]);
    D=exp((j*2*pi*(N-1)/K)*[0:K-1]');
    
    
    Fa=real(D.*Fa);
    giaa_den=[Fa(1);Fa(end:-1:2)];
  end
  gff=giaa_num./giaa_den;
  giaaf=abs(gff).^2; % IAA spectrum
  q=(ifft(giaaf))*K;
  RR=toeplitz(q,conj(q(1:kn)));
  Rr=RR(mg,:);
  
  [Aq,Bq,cq,A0q,B0q,afq,abq]=clwr_ipg(Rr,kn,qn,xg);
  Ac=[Aq;zeros(kn*(kg-qn),kn)];
  Bc=[zeros(kn*(kg-qn),kn);Bq];
  c=bqn_bgs_i(Ac,Bc,xg);
  
  diag_a=zeros(Ng,1);
  for i=1:kn
    aa=Ac(:,i);
    bb=[zeros(kn,1);Bc(1:end-kn,i)];
    d0=abs(aa).^2-abs(bb).^2;
    d=d0;
    for i1=1:kg
      d=d+[zeros(kn*i1,1);d0(1:end-kn*i1)];
    end
    diag_a=diag_a+d;
  end
  
end

f=giaaf;
