% Script to show how to use the fast BVE methods published in:
%
%   A. Jakobsson, G. O. Glentis and E. Gudmundson, ''Computationally
%   Efficient Time-Recursive IAA-Based Blood Velocity Estimation'', IEEE
%   Transactions on Signal Processing.
%
%   Below are some improvements listed that were discussed in the published
%   paper:
%
%   -v2: Code updated with a mean filter applied to the BIAA-based methods,
%   in order to smooth the spectrogram.
%
%   -v3: The sliding window was not implemented as given by eq. 6 in the 
%   paper. The results were still accurate, though. In this version the
%   sliding window has been updated to agree with eq. 6.
%
% By Erik Gudmundson, March 2012
% Updates: Erik Gudmundson October 2012
%

clear

startTime = cputime;

% Set what methods to use for spectral estimation
%methodsToUse = {'ref','refAve'}
%methodsToUse = {'F-BIAA_2'}
%methodsToUse = {'BR-F-BIAA_2'}
%methodsToUse = {'QN-F-BIAA_2'}
methodsToUse = {'BR-QN-F-BIAA_2'}
%methodsToUse = {'ref','refAve','F-BIAA_2','BR-F-BIAA_2','QN-F-BIAA_2','BR-QN-F-BIAA_2'}

doPlots = 1; %Create the plots
doSave = 0; %Save the results
useSlidingWin = 1; %1-Use the sliding window as in eq. 6; 0-No sliding window

%% Load the data 
dataPath = './extractedData/';
dataFileName = 'FieldIIData_1111110000';
%dataFileName = 'FieldIIData_1111110000_short';
load([dataPath dataFileName])

%% Set paths to code for methods
addpath(genpath('fastCodeToolbox'))

%% Set parameters for algorithms
%The number of frequency points per spectrum
nbrOfFreqs = 500;

%Nbr of blocks per spectrum
nbrOfBlocksPerSpec = 13; 

%Nbr of IAA iterations
nbrIaaIter = 10;

% BIAA-QN parameter, ie. order of AR approximation (must be smaller than 
% the number of sub-blocks used, qn < nbrOfBlocksPerSpec (or <=?))
qn=4; 

%% Compute some of the parameters
nbrOfMethods = length(methodsToUse);

%Length of each block of data
blockLen = length(flowShots);

%Size of the loaded data
[nbrFastSamples,totEmissions] = size(data);

%Number of new samples for each spectrum
dN = blockLen;

%The number of emissions devoted to vel. est per block
shotsPerFlow = length(find(flowShots));
%Repeat pattern for the whole length of the spectrum
flowShotsAll = repmat(flowShots,1,nbrOfBlocksPerSpec);
% Sample time vector 
NVec = find(flowShotsAll);
%The number of emissions where no spectral Doppler signal is available
nbrOfEmptyEmissions = length(find(flowShots==0));

%Number of samples to create one spectrum
nbrOfEmissionsPerSpec = length(flowShotsAll);

%The ratio of the number of emissions used for vel. estimation and the 
%total number of emissions
ratioActiveEmissions = sum(flowShotsAll)/length(flowShotsAll);

%Total number of spectra to compute
nbrOfSpec = floor( (totEmissions-nbrOfBlocksPerSpec*blockLen)/dN ) + 1;

%Name of file of the saved data
saveStr = ['FieldII_' setup.typeOfFlowShots '_' num2str(nbrOfBlocksPerSpec) 'blocks_fp' ...
  num2str(nbrOfFreqs) ]

%Compute the phase shift due to depth
phi = 2*pi*setup.f0/setup.fs;

%Generate the demodulating matrix to apply on the different depths
Dinv = repmat(exp(-1i*phi*(0:nbrFastSamples-1)).',1,nbrOfEmissionsPerSpec);

%Forgetting factor for the mean removal when using the sliding window, used
%for recursive mean estimation
forgettingFact = .95;

%%  Initialize data structures
for k = 1:nbrOfMethods
  
  switch methodsToUse{k}
    case 'BR-F-BIAA_2'
      specEstOld_br = [];     
    case 'BR-QN-F-BIAA_2'
      specEstOld_qn = [];
  end
  
end
allSpectra = zeros(nbrOfFreqs,nbrOfSpec,nbrOfMethods);
timePerMethod = zeros(1,nbrOfMethods);

%% Run the methods
disp('Creating spectra...')
for atIter = 0:nbrOfSpec-1
  
  %% Extract current signal
  sigOrig = data(:,(atIter*dN+1):(atIter*dN + nbrOfEmissionsPerSpec));
  
  %Compute the signal mean
  if atIter == 0
    %First computed mean, no forgetting factor can be applied
    sigMean = mean(sigOrig,2)/ratioActiveEmissions;
  else
    %Update the mean with the new data using a forgetting factor 
    sigMean = forgettingFact*sigMean + (1-forgettingFact)*mean(sigOrig(:,end-dN+1:end-nbrOfEmptyEmissions),2);
  end
  
  %Remove mean from signal
  sigZeroMean = sigOrig - repmat(sigMean,1,nbrOfEmissionsPerSpec);
  %Reinsert zeros that were removed due to mean subtraction
  sigZeroMean = sigZeroMean.*(ones(nbrFastSamples,1)*flowShotsAll);
  
  %Sliding window signal update
  if atIter == 0
    %At first iteration, no sliding window can be applied
    sigSlidingWin = sigZeroMean;
  else
    %Sliding window update of the signal matrix
    sigSlidingWin = [sigSlidingWin(:,dN+1:end) sigZeroMean(:,end-dN+1:end)];
  end
  
  %This will store the signal for the next iteration
  %sig = sigSlidingWin;
  
  if useSlidingWin
    sig = sigSlidingWin;
  else
    sig = sigZeroMean;
  end
  
  %sig = sig - repmat(mean(sig,2)/ratioActiveEmissions,1,nbrOfEmissionsPerSpec);
  %Reinsert zeros that were removed due to mean subtraction
  %sig = sig.*(ones(nbrFastSamples,1)*flowShotsAll);
  
  
  %Get the averaged, demodulated data
  aveSig = mean(Dinv.*sig,1);
  
  %Remove the missing data points from the signal (needed for some of the methods)
  aveSigShort = aveSig(NVec);
  %Turn into col. vec
  aveSigShort = aveSigShort(:);
  
  %% Do the estimation
  for k = 1:nbrOfMethods
    
    switch methodsToUse{k}
      case 'ref'
        tmpTime = cputime;
        tmpSpectrum = refSpectrumWelch(sig,nbrOfFreqs);
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
        
      case 'refAve'
        tmpTime = cputime;
        tmpSpectrum = refSpectrumWelch(aveSig,nbrOfFreqs);        
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
        
      case 'BIAA_2' %Average over fast-time
        tmpTime = cputime;
        tmpSpectrum = abs(fftshift(BIAA( ...
          sig, nbrOfFreqs, phi, NVec, nbrIaaIter, 1))).^2;
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
        
      case 'F-BIAA_2'
        tmpTime = cputime;
        tmpSpectrum = FBIAA_2(...
          aveSigShort,nbrOfEmissionsPerSpec,blockLen,shotsPerFlow,...
          nbrOfFreqs,nbrIaaIter);
        %Do fftshift
        tmpSpectrum = fftshift(tmpSpectrum);
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
      
      case 'BR-F-BIAA_2'
        tmpTime = cputime;
        tmpSpectrum = BR_FBIAA_2(...
          aveSigShort,nbrOfEmissionsPerSpec,blockLen,shotsPerFlow,...
          nbrOfFreqs,nbrIaaIter,specEstOld_br);
        %Store spectrum for next iteration
        specEstOld_br = tmpSpectrum;
        %Do fftshift
        tmpSpectrum = fftshift(tmpSpectrum);
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
          
      case 'QN-F-BIAA_2'
        tmpTime = cputime;
        tmpSpectrum = QN_FBIAA_2(...
          aveSigShort,nbrOfEmissionsPerSpec,blockLen,shotsPerFlow,...
          nbrOfFreqs,nbrIaaIter,qn);
        %Do fftshift
        tmpSpectrum = fftshift(tmpSpectrum);
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
        
      case 'BR-QN-F-BIAA_2'
        tmpTime = cputime;
        tmpSpectrum = BR_QN_FBIAA_2(...
          aveSigShort,nbrOfEmissionsPerSpec,blockLen,shotsPerFlow,...
          nbrOfFreqs,nbrIaaIter,specEstOld_qn,qn);
        %Store spectrum for next iteration
        specEstOld_qn = tmpSpectrum;
        %Do fftshift
        tmpSpectrum = fftshift(tmpSpectrum);
        timePerMethod(k) = timePerMethod(k) + cputime - tmpTime;
        
      otherwise
        error(['Method: ' methodsToUse{k} ' is unknown or not implemented!'])
        
    end %of switch
    
    %Store the current spectrum
    allSpectra(:,atIter+1,k) = tmpSpectrum;
    
  end %of nbrOfMethods
  
  if (rem(atIter,round(nbrOfSpec/20))==0)
    disp([num2str(round(atIter/nbrOfSpec*100)),'% done'])
  end
  
end %of atIter

%Compute time taken for estimation
timeTaken = cputime - startTime;
disp(['Time taken: ' num2str(timeTaken/60) ' min'])
timePerMethod


clear specEstOld_br specEstOld_qn sig
clear Dinv atIter aveSig aveSigShort k
clear ratioActiveEmissions startTime tmpTime

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plot the estimated spectra

%Generate frequency and time axes
timeAxis = (0:nbrOfSpec-1)*dN/setup.fprf;
freqVec = setup.fprf/nbrOfFreqs*(-(nbrOfFreqs/2):(nbrOfFreqs/2-1));
velVec = setup.c/2*freqVec/setup.f0/cos(setup.theta);

%Scale power density
dB_range = 40;
%dB_range = 50;
%dB_range = 60;

if doPlots
  
  %Create a window vector for mean filtering of BIAA method in order
  %to make it smoother
  winVecTmp = linspace(0,1,5);
  winVec = [winVecTmp(2:end) winVecTmp(end-1:-1:2)];
  
  flipData = 0;
  figHandles = zeros(1,nbrOfMethods);
  %Plot each method in a separate plot
  for k = 1:nbrOfMethods
    
    if ~isempty(regexp(methodsToUse{k},'BIAA','once')) 
      %Filter the BIAA data to make it smoother
      allSpectra(:,:,k) = meanFilter(allSpectra(:,:,k),winVec);
      disp('Applied mean filter to the data...')
    end
    
    %Set method name as title for plot
    extra.figTitle = [setup.typeOfFlowShots ', alg: ' methodsToUse{k}];
    figHandles(k) = plotSpectrogram(...
      allSpectra(:,:,k),timeAxis,velVec,dB_range,flipData,extra);
  end
  
  %Save figures as png-plots
  if 0
    for k = 1:nbrOfMethods
      %currFig = figHandles(k)
      saveas(figHandles(k),[saveStr '_' methodsToUse{k} '.png'])
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if doSave
  clear doSave doPlots
  
  %Remove the loaded data
  clear data
  clear figHandles k flipData
  
  disp('Saving the results...')
  
  savePrefix = './savedProcessedData/';
  saveDatePrefix = [datestr(date,29) '/'];
  
  if (~exist([savePrefix saveDatePrefix],'dir'))
    disp(['Creating folder ' [savePrefix saveDatePrefix] '...'])
    eval(['mkdir ' savePrefix saveDatePrefix])
  end
  dateAndTime = datestr(clock,30); %ISO 8601 time format
  %Unique file name
  save([savePrefix saveDatePrefix saveStr '_' dateAndTime])
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
